@@ -1,10 +1,11 @@
-// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
 package main
 
 import (
+	"bytes"
 	"fmt"
 	"os"
 	"os/exec"
@@ -19,24 +20,27 @@ import (
 	"github.com/blang/semver"
 	"github.com/gcla/gowid"
 	"github.com/gcla/termshark/v2"
-	"github.com/gcla/termshark/v2/capinfo"
-	"github.com/gcla/termshark/v2/cli"
-	"github.com/gcla/termshark/v2/convs"
-	"github.com/gcla/termshark/v2/pcap"
-	"github.com/gcla/termshark/v2/shark"
-	"github.com/gcla/termshark/v2/streams"
-	"github.com/gcla/termshark/v2/system"
-	"github.com/gcla/termshark/v2/theme"
-	"github.com/gcla/termshark/v2/tty"
+	"github.com/gcla/termshark/v2/configs/profiles"
+	"github.com/gcla/termshark/v2/pkg/capinfo"
+	"github.com/gcla/termshark/v2/pkg/cli"
+	"github.com/gcla/termshark/v2/pkg/confwatcher"
+	"github.com/gcla/termshark/v2/pkg/convs"
+	"github.com/gcla/termshark/v2/pkg/fields"
+	"github.com/gcla/termshark/v2/pkg/pcap"
+	"github.com/gcla/termshark/v2/pkg/shark"
+	"github.com/gcla/termshark/v2/pkg/streams"
+	"github.com/gcla/termshark/v2/pkg/summary"
+	"github.com/gcla/termshark/v2/pkg/system"
+	"github.com/gcla/termshark/v2/pkg/tailfile"
+	"github.com/gcla/termshark/v2/pkg/tty"
 	"github.com/gcla/termshark/v2/ui"
 	"github.com/gcla/termshark/v2/widgets/filter"
 	"github.com/gcla/termshark/v2/widgets/wormhole"
-	"github.com/gdamore/tcell"
+	"github.com/gdamore/tcell/v2"
 	flags "github.com/jessevdk/go-flags"
 	"github.com/mattn/go-isatty"
 	"github.com/shibukawa/configdir"
 	log "github.com/sirupsen/logrus"
-	"github.com/spf13/viper"
 
 	"net/http"
 	_ "net/http"
@@ -54,13 +58,14 @@ func main() {
 	// stopped. Any exception is a bug.
 	var ensureGoroutinesStopWG sync.WaitGroup
 	filter.Goroutinewg = &ensureGoroutinesStopWG
-	termshark.Goroutinewg = &ensureGoroutinesStopWG
 	pcap.Goroutinewg = &ensureGoroutinesStopWG
 	streams.Goroutinewg = &ensureGoroutinesStopWG
 	capinfo.Goroutinewg = &ensureGoroutinesStopWG
 	convs.Goroutinewg = &ensureGoroutinesStopWG
 	ui.Goroutinewg = &ensureGoroutinesStopWG
 	wormhole.Goroutinewg = &ensureGoroutinesStopWG
+	summary.Goroutinewg = &ensureGoroutinesStopWG
+	confwatcher.Goroutinewg = &ensureGoroutinesStopWG
 
 	res := cmain()
 	ensureGoroutinesStopWG.Wait()
@@ -133,8 +138,6 @@ func cmain() int {
 	// processes start running, they use the same tty line discipline.
 	system.RegisterForSignals(sigChan)
 
-	viper.SetConfigName("termshark") // no need to include file extension - looks for file called termshark.ini for example
-
 	stdConf := configdir.New("", "termshark")
 	dirs := stdConf.QueryFolders(configdir.Cache)
 	if err := dirs[0].CreateParentDir("dummy"); err != nil {
@@ -143,29 +146,17 @@ func cmain() int {
 	dirs = stdConf.QueryFolders(configdir.Global)
 	if err := dirs[0].CreateParentDir("dummy"); err != nil {
 		fmt.Fprintf(os.Stderr, "Warning: could not create config dir: %v\n", err)
-	}
-	viper.AddConfigPath(dirs[0].Path)
-
-	if f, err := os.OpenFile(filepath.Join(dirs[0].Path, "termshark.toml"), os.O_RDONLY|os.O_CREATE, 0666); err != nil {
-		fmt.Fprintf(os.Stderr, "Warning: could not create initial config file: %v\n", err)
 	} else {
-		f.Close()
-	}
-
-	err := viper.ReadInConfig()
-	if err != nil {
-		fmt.Println("Config file not found...")
-	}
-
-	if os.Getenv("TERMSHARK_CAPTURE_MODE") == "1" {
-		err = system.DumpcapExt(termshark.DumpcapBin(), termshark.TSharkBin(), os.Args[1:]...)
-		if err != nil {
-			return 1
-		} else {
-			return 0
+		if err = os.MkdirAll(filepath.Join(dirs[0].Path, "profiles"), 0755); err != nil {
+			fmt.Fprintf(os.Stderr, "Warning: could not create profiles dir: %v\n", err)
 		}
 	}
 
+	err := profiles.ReadDefaultConfig(dirs[0].Path)
+	if err != nil {
+		fmt.Fprintf(os.Stderr, fmt.Sprintf("%s\n", err.Error()))
+	}
+
 	// Used to determine if we should run tshark instead e.g. stdout is not a tty
 	var tsopts cli.Tshark
 
@@ -192,7 +183,7 @@ func cmain() int {
 	// On Windows, termshark itself is used to tail the pcap generated by dumpcap, and the output
 	// is fed into tshark -T psml ...
 	if tsopts.TailFileValue() != "" {
-		err = termshark.TailFile(tsopts.TailFileValue())
+		err = tailfile.Tail(tsopts.TailFileValue())
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "Error: %v", err)
 			return 1
@@ -201,6 +192,29 @@ func cmain() int {
 		}
 	}
 
+	// From here, the current profile is referenced. So load it up prior to first use. If the user
+	// provides a non-existent profile name, it should be an error, just as for Wireshark.
+	if tsopts.Profile != "" {
+		if err = profiles.Use(tsopts.Profile); err != nil {
+			fmt.Fprintf(os.Stderr, "Error: %v\n", err)
+			return 1
+		}
+	}
+
+	// If this variable is set, it's set by termshark internally, and termshark is guaranteed to
+	// construct a valid command-line invocation. So it doesn't matter if I do this after the CLI
+	// parsing logic because there's no risk of an error causing a short-circuit and this command
+	// not being run. The reason to do it after the CLI parsing logic is so that I have the correct
+	// config profile loaded, needed for the tshark command.
+	if os.Getenv("TERMSHARK_CAPTURE_MODE") == "1" {
+		err = system.DumpcapExt(termshark.DumpcapBin(), termshark.TSharkBin(), os.Args[1:]...)
+		if err != nil {
+			return 1
+		} else {
+			return 0
+		}
+	}
+
 	// Run after accessing the config so I can use the configured tshark binary, if there is one. I need that
 	// binary in the case that termshark is run where stdout is not a tty, in which case I exec tshark - but
 	// it makes sense to use the one in termshark.toml
@@ -307,12 +321,11 @@ func cmain() int {
 		// using the gcla fork of tcell for another application).
 		usetty = "/dev/tty"
 	}
-	os.Setenv("GOWID_TTY", usetty)
 
 	// Allow the user to override the shell's TERM variable this way. Perhaps the user runs
 	// under screen/tmux, and the TERM variable doesn't reflect the fact their preferred
 	// terminal emumlator supports 256 colors.
-	termVar := termshark.ConfString("main.term", "")
+	termVar := profiles.ConfString("main.term", "")
 	if termVar != "" {
 		fmt.Fprintf(os.Stderr, "Configuration file overrides TERM setting, using TERM=%s\n", termVar)
 		os.Setenv("TERM", termVar)
@@ -533,7 +546,7 @@ func cmain() int {
 	}
 
 	debug := false
-	if (opts.Debug.Set && opts.Debug.Val == true) || (!opts.Debug.Set && termshark.ConfBool("main.debug", false)) {
+	if (opts.Debug.Set && opts.Debug.Val == true) || (!opts.Debug.Set && profiles.ConfBool("main.debug", false)) {
 		debug = true
 	}
 
@@ -575,7 +588,7 @@ func cmain() int {
 	// Here, tsharkBin is a fully-qualified tshark binary that exists on the fs (absent race
 	// conditions...)
 
-	valids := termshark.ConfStrings("main.validated-tsharks")
+	valids := profiles.ConfStrings("main.validated-tsharks")
 
 	if !termshark.StringInSlice(tsharkBin, valids) {
 		tver, err := termshark.TSharkVersion(tsharkBin)
@@ -592,24 +605,24 @@ func cmain() int {
 		}
 
 		valids = append(valids, tsharkBin)
-		termshark.SetConf("main.validated-tsharks", valids)
+		profiles.SetConf("main.validated-tsharks", valids)
 	}
 
 	// If the last tshark we used isn't the same as the current one, then remove the cached fields
 	// data structure so it can be regenerated.
-	if tsharkBin != termshark.ConfString("main.last-used-tshark", "") {
-		termshark.DeleteCachedFields()
+	if tsharkBin != profiles.ConfString("main.last-used-tshark", "") {
+		fields.DeleteCachedFields()
 	}
 
 	// Write out the last-used tshark path. We do this to make the above fields cache be consistent
 	// with the tshark binary we're using.
-	termshark.SetConf("main.last-used-tshark", tsharkBin)
+	profiles.SetConf("main.last-used-tshark", tsharkBin)
 
 	// Determine if the current binary supports color. Tshark will fail with an error if it's too old
 	// and you supply the --color flag. Assume true, and check if our current binary is not in the
 	// validate list.
 	ui.PacketColorsSupported = true
-	colorTsharks := termshark.ConfStrings("main.color-tsharks")
+	colorTsharks := profiles.ConfStrings("main.color-tsharks")
 
 	if !termshark.StringInSlice(tsharkBin, colorTsharks) {
 		ui.PacketColorsSupported, err = termshark.TSharkSupportsColor(tsharkBin)
@@ -617,7 +630,7 @@ func cmain() int {
 			ui.PacketColorsSupported = false
 		} else {
 			colorTsharks = append(colorTsharks, tsharkBin)
-			termshark.SetConf("main.color-tsharks", colorTsharks)
+			profiles.SetConf("main.color-tsharks", colorTsharks)
 		}
 	}
 
@@ -684,7 +697,7 @@ func cmain() int {
 		}
 	}
 
-	watcher, err := termshark.NewConfigWatcher()
+	watcher, err := confwatcher.New()
 	if err != nil {
 		fmt.Fprintf(os.Stderr, "Problem constructing config file watcher: %v", err)
 		return 1
@@ -712,6 +725,7 @@ func cmain() int {
 	//======================================================================
 
 	ifaceExitCode := 0
+	stderr := &bytes.Buffer{}
 	var ifaceErr error
 
 	// This is deferred until after the app is Closed - otherwise messages written to stdout/stderr are
@@ -723,6 +737,20 @@ func cmain() int {
 				fmt.Fprintf(os.Stderr, ": %v", ifaceErr)
 			}
 			fmt.Fprintf(os.Stderr, " (exit code %d)\n", ifaceExitCode)
+			if stderr.Len() != 0 {
+				// The default capture bin is termshark itself, with a special environment
+				// variable set that causes it to try dumpcap, then tshark, in that order (for
+				// efficiency of capture, but falling back to tshark for extcap interfaces).
+				// But telling the user the capture process is "termshark" is misleading.
+				cbin, err1 := filepath.Abs(filepath.FromSlash(termshark.CaptureBin()))
+				def, err2 := filepath.Abs("termshark")
+				if err1 == nil && err2 == nil && cbin == def {
+					cbin = "the capture process"
+				}
+
+				fmt.Fprintf(os.Stderr, "Standard error stream from %s:\n", cbin)
+				fmt.Fprintf(os.Stderr, "------\n%s\n------\n", stderr.String())
+			}
 			if runtime.GOOS == "linux" && os.Geteuid() != 0 {
 				fmt.Fprintf(os.Stderr, "You might need: sudo setcap cap_net_raw,cap_net_admin+eip %s\n", termshark.PrivilegedBin())
 				fmt.Fprintf(os.Stderr, "Or try running with sudo or as root.\n")
@@ -732,24 +760,24 @@ func cmain() int {
 	}()
 
 	// Initialize application state for dark mode and auto-scroll
-	ui.DarkMode = termshark.ConfBool("main.dark-mode", false)
-	ui.AutoScroll = termshark.ConfBool("main.auto-scroll", true)
-	ui.PacketColors = termshark.ConfBool("main.packet-colors", true)
+	ui.DarkMode = profiles.ConfBool("main.dark-mode", true)
+	ui.AutoScroll = profiles.ConfBool("main.auto-scroll", true)
+	ui.PacketColors = profiles.ConfBool("main.packet-colors", true)
 
 	// Set them up here so they have access to any command-line flags that
 	// need to be passed to the tshark commands used
-	pdmlArgs := termshark.ConfStringSlice("main.pdml-args", []string{})
-	psmlArgs := termshark.ConfStringSlice("main.psml-args", []string{})
+	pdmlArgs := profiles.ConfStringSlice("main.pdml-args", []string{})
+	psmlArgs := profiles.ConfStringSlice("main.psml-args", []string{})
 	if opts.TimestampFormat != "" {
 		psmlArgs = append(psmlArgs, "-t", opts.TimestampFormat)
 	}
-	tsharkArgs := termshark.ConfStringSlice("main.tshark-args", []string{})
+	tsharkArgs := profiles.ConfStringSlice("main.tshark-args", []string{})
 	if ui.PacketColors && !ui.PacketColorsSupported {
 		log.Warnf("Packet coloring is enabled, but %s does not support --color", tsharkBin)
 		ui.PacketColors = false
 	}
-	cacheSize := termshark.ConfInt("main.pcap-cache-size", 64)
-	bundleSize := termshark.ConfInt("main.pcap-bundle-size", 1000)
+	cacheSize := profiles.ConfInt("main.pcap-cache-size", 64)
+	bundleSize := profiles.ConfInt("main.pcap-bundle-size", 1000)
 	if bundleSize <= 0 {
 		maxBundleSize := 100000
 		log.Infof("Config specifies pcap-bundle-size as %d - setting to max (%d)", bundleSize, maxBundleSize)
@@ -795,8 +823,8 @@ func cmain() int {
 	// 0-21 are set up normally, and not remapped. If the closest match is one of those
 	// colors, then the theme won't look as expected. A workaround is to tell
 	// gowid not to use colors 0-21 when finding the closest match.
-	if termshark.ConfKeyExists("main.ignore-base16-colors") {
-		gowid.IgnoreBase16 = termshark.ConfBool("main.ignore-base16-colors", false)
+	if profiles.ConfKeyExists("main.ignore-base16-colors") {
+		gowid.IgnoreBase16 = profiles.ConfBool("main.ignore-base16-colors", false)
 	} else {
 		// Try to auto-detect whether or not base16-shell is installed and in-use
 		gowid.IgnoreBase16 = (os.Getenv("BASE16_SHELL") != "")
@@ -810,12 +838,15 @@ func cmain() int {
 		// just an implementation snafu, or if something else is going on... In any case,
 		// termshark will fall back to 256-colors if base16 is detected because I can
 		// programmatically avoid choosing colors 0-21 for anything termshark needs.
-		if os.Getenv("COLORTERM") != "" && !termshark.ConfBool("main.respect-colorterm", false) {
+		if os.Getenv("COLORTERM") != "" && !profiles.ConfBool("main.respect-colorterm", false) {
 			log.Infof("Pessimistically disabling 24-bit color to avoid conflicts with base16")
 			os.Unsetenv("COLORTERM")
 		}
 	}
 
+	// the app variable is created here so I can bind it in the defer below
+	var app *gowid.App
+
 	// Do this before ui.Build. If ui.Build fails (e.g. bad TERM), then the filter will be left
 	// running, so we need the defer to be in effect here and not after the processing of ui.Build's
 	// error
@@ -823,10 +854,18 @@ func cmain() int {
 		if ui.FilterWidget != nil {
 			ui.FilterWidget.Close()
 		}
+		if ui.SearchWidget != nil {
+			ui.SearchWidget.Close(app)
+		}
+		if ui.CurrentWormholeWidget != nil {
+			ui.CurrentWormholeWidget.Close()
+		}
+		if ui.CurrentColsWidget != nil {
+			ui.CurrentColsWidget.Close()
+		}
 	}()
 
-	var app *gowid.App
-	if app, err = ui.Build(); err != nil {
+	if app, err = ui.Build(usetty); err != nil {
 		fmt.Fprintf(os.Stderr, "Error: %v\n", err)
 		// Tcell returns ExitError now because if its internal terminfo DB does not have
 		// a matching entry, it tries to build one with infocmp.
@@ -853,7 +892,7 @@ func cmain() int {
 
 	ui.Running = false
 
-	validator := filter.Validator{
+	validator := filter.DisplayFilterValidator{
 		Invalid: &filter.ValidateCB{
 			App: app,
 			Fn: func(app gowid.IApp) {
@@ -904,6 +943,7 @@ func cmain() int {
 			ui.RequestLoadPcap(absfile, displayFilter, ui.NoGlobalJump, app)
 		}
 		validator.Valid = &filter.ValidateCB{Fn: doit, App: app}
+		validator.EmptyCB = &filter.ValidateCB{Fn: doit, App: app}
 		validator.Validate(displayFilter)
 	} else {
 
@@ -911,10 +951,11 @@ func cmain() int {
 		ifaceExitCode = 0
 		for _, psrc := range psrcs {
 			if psrc.IsInterface() {
-				if ifaceExitCode, ifaceErr = termshark.RunForExitCode(
+				if ifaceExitCode, ifaceErr = termshark.RunForStderr(
 					termshark.CaptureBin(),
 					[]string{"-i", psrc.Name(), "-a", "duration:1"},
 					append(os.Environ(), "TERMSHARK_CAPTURE_MODE=1"),
+					stderr,
 				); ifaceExitCode != 0 {
 					return 1
 				}
@@ -925,14 +966,20 @@ func cmain() int {
 			}
 		}
 
+		doLoad := func(app gowid.IApp) {
+			ifacePcapFilename = ifaceTmpFile
+			ui.RequestLoadInterfaces(psrcs, captureFilter, displayFilter, ifaceTmpFile, app)
+		}
+
 		ifValid := func(app gowid.IApp) {
 			app.Run(gowid.RunFunction(func(app gowid.IApp) {
 				ui.FilterWidget.SetValue(displayFilter, app)
 			}))
-			ifacePcapFilename = ifaceTmpFile
-			ui.RequestLoadInterfaces(psrcs, captureFilter, displayFilter, ifaceTmpFile, app)
+			doLoad(app)
 		}
+
 		validator.Valid = &filter.ValidateCB{Fn: ifValid, App: app}
+		validator.EmptyCB = &filter.ValidateCB{Fn: doLoad, App: app}
 		validator.Validate(displayFilter)
 	}
 
@@ -962,6 +1009,7 @@ func cmain() int {
 
 	inactiveDuration := 60 * time.Second
 	inactivityTimer := time.NewTimer(inactiveDuration)
+	currentlyInactive := false // True if the timer has fired and termshark is in "inactive" state
 
 	var progCancelTimer *time.Timer
 
@@ -1000,8 +1048,8 @@ Loop:
 				// loads/filters to that now.
 				ui.Loader.TurnOffPipe()
 				app.Run(gowid.RunFunction(func(app gowid.IApp) {
-					ui.ClearProgressWidget(app)
-					ui.SetProgressDeterminate(app) // always switch back - for pdml (partial) loads of later data.
+					ui.ClearProgressWidgetFor(app, ui.LoaderOwns)
+					ui.SetProgressDeterminateFor(app, ui.LoaderOwns) // always switch back - for pdml (partial) loads of later data.
 				}))
 
 				// When the progress bar is enabled, track the previous percentage reached. This is
@@ -1042,7 +1090,7 @@ Loop:
 				if progCancelTimer == nil {
 					progCancelTimer = time.AfterFunc(time.Duration(100000)*time.Hour, func() {
 						app.Run(gowid.RunFunction(func(app gowid.IApp) {
-							ui.ClearProgressWidget(app)
+							ui.ClearProgressWidgetFor(app, ui.LoaderOwns)
 							progCancelTimer = nil
 						}))
 					})
@@ -1074,7 +1122,7 @@ Loop:
 			}
 		}
 
-		if ui.Loader.InterfaceLoader.IsLoading() {
+		if ui.Loader.InterfaceLoader.IsLoading() && !currentlyInactive {
 			inactivityChan = inactivityTimer.C
 		}
 
@@ -1110,7 +1158,7 @@ Loop:
 		case <-checkPcapCacheChan:
 			// Only check the cache dir if we own it; don't want to delete pcap files
 			// that might be shared with wireshark
-			if termshark.ConfBool("main.use-tshark-temp-for-pcap-cache", false) {
+			if profiles.ConfBool("main.use-tshark-temp-for-pcap-cache", false) {
 				log.Infof("Termshark does not own the pcap temp dir %s; skipping size check", termshark.PcapDir())
 			} else {
 				termshark.PrunePcapCache()
@@ -1118,8 +1166,10 @@ Loop:
 			checkedPcapCache = true
 
 		case <-inactivityChan:
-			ui.Fin.Activate()
-			app.Redraw()
+			if ui.Fin != nil {
+				ui.Fin.Activate()
+			}
+			currentlyInactive = true
 
 		case <-finChan:
 			ui.Fin.Advance()
@@ -1137,24 +1187,7 @@ Loop:
 			// Need to do that here because the app won't know how many colors the screen
 			// has (and therefore which variant of the theme to load) until the screen is
 			// activated.
-			mode := app.GetColorMode()
-			modeStr := theme.Mode(mode) // more concise
-			themeName := termshark.ConfString(fmt.Sprintf("main.theme-%s", modeStr), "default")
-			loaded := false
-			if themeName != "" {
-				err = theme.Load(themeName, app)
-				if err != nil {
-					log.Warnf("Theme %s could not be loaded: %v", themeName, err)
-				} else {
-					loaded = true
-				}
-			}
-			if !loaded && themeName != "default" {
-				err = theme.Load("default", app)
-				if err != nil {
-					log.Warnf("Theme %s could not be loaded: %v", themeName, err)
-				}
-			}
+			ui.ApplyCurrentTheme(app)
 
 			// This needs to run after the toml config file is loaded.
 			ui.SetupColors()
@@ -1163,7 +1196,7 @@ Loop:
 			appRunner.Start()
 
 			// Reinstate  our terminal overrides that allow ctrl-z
-			if err := ctrlzLineDisc.Set(); err != nil {
+			if err := ctrlzLineDisc.Set(usetty); err != nil {
 				ui.OpenError(fmt.Sprintf("Unexpected error setting Ctrl-z handler: %v\n", err), app)
 			}
 
@@ -1173,11 +1206,11 @@ Loop:
 			ui.StartUIChan = nil // make sure it's not triggered again
 
 			if runtime.GOOS != "windows" {
-				if mode == gowid.Mode8Colors {
+				if app.GetColorMode() == gowid.Mode8Colors {
 					// If exists is true, it means we already tried and then reverted back, so
 					// just load up termshark normally with no further interruption.
 					if _, exists := os.LookupEnv("TERMSHARK_ORIGINAL_TERM"); !exists {
-						if !termshark.ConfBool("main.disable-term-helper", false) {
+						if !profiles.ConfBool("main.disable-term-helper", false) {
 							err = termshark.Does256ColorTermExist()
 							if err != nil {
 								log.Infof("Must use 8-color mode because 256-color version of TERM=%s unavailable - %v.", os.Getenv("TERM"), err)
@@ -1225,12 +1258,6 @@ Loop:
 				// will happen next time round because the quitRequested flag is checked.
 				stopLoaders()
 			}
-			if ui.CurrentWormholeWidget != nil {
-				ui.CurrentWormholeWidget.Close()
-			}
-			if ui.CurrentColsWidget != nil {
-				ui.CurrentColsWidget.Close()
-			}
 
 		case sig := <-sigChan:
 			if system.IsSigTSTP(sig) {
@@ -1267,7 +1294,7 @@ Loop:
 					appRunner.Start()
 
 					// Reinstate  our terminal overrides that allow ctrl-z
-					if err := ctrlzLineDisc.Set(); err != nil {
+					if err := ctrlzLineDisc.Set(usetty); err != nil {
 						ui.OpenError(fmt.Sprintf("Unexpected error setting Ctrl-z handler: %v\n", err), app)
 					}
 
@@ -1312,6 +1339,7 @@ Loop:
 		case ev := <-tcellEvents:
 			app.HandleTCellEvent(ev, gowid.IgnoreUnhandledInput)
 			inactivityTimer.Reset(inactiveDuration)
+			currentlyInactive = false
 			checkPcapCacheTimer.Reset(checkPcapCacheDuration)
 
 		case ev, ok := <-afterRenderEvents:
@@ -1321,11 +1349,14 @@ Loop:
 			if !ok {
 				break Loop
 			}
-			app.RunThenRenderEvent(ev)
+			ev.RunThenRenderEvent(app)
+			if ui.Running {
+				app.RedrawTerminal()
+			}
 
 		case <-watcher.ConfigChanged():
 			// Re-read so changes that can take effect immediately do so
-			if err := viper.ReadInConfig(); err != nil {
+			if err := profiles.ReadDefaultConfig(dirs[0].Path); err != nil {
 				log.Warnf("Unexpected error re-reading toml config: %v", err)
 			}
 			ui.UpdateRecentMenu(app)
