@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -31,9 +31,10 @@ import (
 	"github.com/gcla/gowid/widgets/text"
 	"github.com/gcla/gowid/widgets/vpadding"
 	"github.com/gcla/termshark/v2"
-	"github.com/gcla/termshark/v2/convs"
-	"github.com/gcla/termshark/v2/pcap"
-	"github.com/gcla/termshark/v2/psmlmodel"
+	"github.com/gcla/termshark/v2/configs/profiles"
+	"github.com/gcla/termshark/v2/pkg/convs"
+	"github.com/gcla/termshark/v2/pkg/pcap"
+	"github.com/gcla/termshark/v2/pkg/psmlmodel"
 	"github.com/gcla/termshark/v2/ui/tableutil"
 	"github.com/gcla/termshark/v2/widgets/appkeys"
 	"github.com/gcla/termshark/v2/widgets/copymodetable"
@@ -41,7 +42,7 @@ import (
 	"github.com/gcla/termshark/v2/widgets/keepselected"
 	"github.com/gcla/termshark/v2/widgets/scrollabletable"
 	"github.com/gcla/termshark/v2/widgets/withscrollbar"
-	"github.com/gdamore/tcell"
+	"github.com/gdamore/tcell/v2"
 )
 
 var convsView *holder.Widget
@@ -61,6 +62,14 @@ const (
 	From Direction = iota
 )
 
+type ConvAddr int
+
+const (
+	IPv4Addr ConvAddr = 0
+	IPv6Addr ConvAddr = iota
+	MacAddr  ConvAddr = iota
+)
+
 type FilterMask int
 
 const (
@@ -318,19 +327,19 @@ type ConvsUiWidget struct {
 }
 
 func (w *ConvsUiWidget) AbsoluteTime() bool {
-	return termshark.ConfBool("main.conv-absolute-time", false)
+	return profiles.ConfBool("main.conv-absolute-time", false)
 }
 
 func (w *ConvsUiWidget) SetAbsoluteTime(val bool) {
-	termshark.SetConf("main.conv-absolute-time", val)
+	profiles.SetConf("main.conv-absolute-time", val)
 }
 
 func (w *ConvsUiWidget) ResolveNames() bool {
-	return termshark.ConfBool("main.conv-resolve-names", false)
+	return profiles.ConfBool("main.conv-resolve-names", false)
 }
 
 func (w *ConvsUiWidget) SetResolveNames(val bool) {
-	termshark.SetConf("main.conv-resolve-names", val)
+	profiles.SetConf("main.conv-resolve-names", val)
 }
 
 func (w *ConvsUiWidget) Context() context.Context {
@@ -342,11 +351,11 @@ func (w *ConvsUiWidget) FilterValue() string {
 }
 
 func (w *ConvsUiWidget) UseFilter() bool {
-	return termshark.ConfBool("main.conv-use-filter", false)
+	return profiles.ConfBool("main.conv-use-filter", false)
 }
 
 func (w *ConvsUiWidget) SetUseFilter(val bool) {
-	termshark.SetConf("main.conv-use-filter", val)
+	profiles.SetConf("main.conv-use-filter", val)
 }
 
 func (w *ConvsUiWidget) construct() {
@@ -833,6 +842,13 @@ func (w *ConvsUiWidget) OnData(data string, app gowid.IApp) {
 			ports = termshark.StringInSlice(cur, []string{"UDP", "TCP"})
 			ipv6 := (cur == "IPv6")
 
+			var addrComp table.ICompare = termshark.IPCompare{}
+			if termshark.StringInSlice(cur, []string{"Ethernet"}) {
+				addrComp = termshark.MACCompare{}
+			}
+
+			var convComp table.ICompare = termshark.ConvPktsCompare{}
+
 			if ports {
 				hdrs = []string{
 					"Addr A",
@@ -863,16 +879,16 @@ func (w *ConvsUiWidget) OnData(data string, app gowid.IApp) {
 					weightupto(200, 8),  // durn
 				}
 				comps = []table.ICompare{
-					table.StringCompare{},
-					table.IntCompare{},
-					table.StringCompare{},
-					table.IntCompare{},
-					table.IntCompare{},
+					addrComp,
 					table.IntCompare{},
+					addrComp,
 					table.IntCompare{},
 					table.IntCompare{},
+					convComp,
 					table.IntCompare{},
+					convComp,
 					table.IntCompare{},
+					convComp,
 					table.FloatCompare{},
 					table.FloatCompare{},
 				}
@@ -908,14 +924,14 @@ func (w *ConvsUiWidget) OnData(data string, app gowid.IApp) {
 					wids[1] = weightupto(500, 42)
 				}
 				comps = []table.ICompare{
-					table.StringCompare{},
-					table.StringCompare{},
-					table.IntCompare{},
-					table.IntCompare{},
-					table.IntCompare{},
+					addrComp,
+					addrComp,
 					table.IntCompare{},
+					convComp,
 					table.IntCompare{},
+					convComp,
 					table.IntCompare{},
+					convComp,
 					table.FloatCompare{},
 					table.FloatCompare{},
 				}
@@ -925,6 +941,10 @@ func (w *ConvsUiWidget) OnData(data string, app gowid.IApp) {
 			continue
 		}
 
+		line = strings.Replace(line, " bytes", "", -1)
+		line = strings.Replace(line, "bytes", "", -1)
+		line = strings.Replace(line, " kB", "kB", -1)
+		line = strings.Replace(line, " MB", "MB", -1)
 		r = strings.NewReader(line)
 		n, err = fmt.Fscanf(r, "%s <-> %s %s %s %s %s %s %s %s %s",
 			&addra,
@@ -939,6 +959,12 @@ func (w *ConvsUiWidget) OnData(data string, app gowid.IApp) {
 			&durn,
 		)
 		if err == nil && n == 10 {
+			bytesto = strings.Replace(bytesto, "kB", " kB", -1)
+			bytesfrom = strings.Replace(bytesfrom, "kB", " kB", -1)
+			bytes = strings.Replace(bytes, "kB", " kB", -1)
+			bytesto = strings.Replace(bytesto, "MB", " MB", -1)
+			bytesfrom = strings.Replace(bytesfrom, "MB", " MB", -1)
+			bytes = strings.Replace(bytes, "MB", " MB", -1)
 			if ports {
 				pa := strings.Split(addra, ":")
 				pb := strings.Split(addrb, ":")
