@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -14,7 +14,7 @@ import (
 func openLogsUi(app gowid.IApp) {
 }
 
-func closeLogsUi(app gowid.IApp) {
+func openConfigUi(app gowid.IApp) {
 }
 
 //======================================================================
