@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -23,7 +23,7 @@ import (
 	"github.com/gcla/gowid/widgets/text"
 	"github.com/gcla/gowid/widgets/vpadding"
 	"github.com/gcla/termshark/v2/widgets/appkeys"
-	"github.com/gdamore/tcell"
+	"github.com/gdamore/tcell/v2"
 )
 
 //======================================================================
