@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -8,13 +8,13 @@ package ui
 import (
 	"fmt"
 	"os"
-	"strings"
 	"time"
 
 	"github.com/gcla/gowid"
 	"github.com/gcla/gowid/widgets/table"
 	"github.com/gcla/termshark/v2"
-	"github.com/gcla/termshark/v2/pcap"
+	"github.com/gcla/termshark/v2/configs/profiles"
+	"github.com/gcla/termshark/v2/pkg/pcap"
 	log "github.com/sirupsen/logrus"
 )
 
@@ -71,6 +71,9 @@ func MakePacketViewUpdater() updatePacketViews {
 
 func (t updatePacketViews) OnClear(code pcap.HandlerCode, app gowid.IApp) {
 	clearPacketViews(app)
+	if packetListView != nil {
+		updatePacketListWithData(t.Ld, app)
+	}
 }
 
 func (t updatePacketViews) BeforeBegin(code pcap.HandlerCode, app gowid.IApp) {
@@ -117,15 +120,10 @@ func (t updatePacketViews) OnError(code pcap.HandlerCode, app gowid.IApp, err er
 		fmt.Fprintf(os.Stderr, "%v\n", err)
 		RequestQuit()
 	} else {
-		if !termshark.ConfBool("main.suppress-tshark-errors", false) {
+		if !profiles.ConfBool("main.suppress-tshark-errors", true) {
 			var errstr string
 			if kverr, ok := err.(gowid.KeyValueError); ok {
-				errstr = fmt.Sprintf("%v\n\n", kverr.Cause())
-				kvs := make([]string, 0, len(kverr.KeyVals))
-				for k, v := range kverr.KeyVals {
-					kvs = append(kvs, fmt.Sprintf("%v: %v", k, v))
-				}
-				errstr = errstr + strings.Join(kvs, "\n")
+				errstr = termshark.KeyValueErrorString(kverr)
 			} else {
 				errstr = fmt.Sprintf("%v", err)
 			}
@@ -150,7 +148,7 @@ func (t SimpleErrors) OnError(code pcap.HandlerCode, app gowid.IApp, err error)
 	log.Error(err)
 	// Hack to avoid picking up errors at other parts of the load
 	// cycle. There should be specific handlers for specific errors.
-	if !termshark.ConfBool("main.suppress-tshark-errors", false) {
+	if !profiles.ConfBool("main.suppress-tshark-errors", true) {
 		app.Run(gowid.RunFunction(func(app gowid.IApp) {
 			OpenError(fmt.Sprintf("%v", err), app)
 		}))
@@ -256,6 +254,28 @@ func (t checkGlobalJumpAfterPsml) OnClear(code pcap.HandlerCode, app gowid.IApp)
 
 //======================================================================
 
+func clearSearchData(app gowid.IApp) {
+	if SearchWidget != nil {
+		SearchWidget.Clear(app)
+	}
+}
+
+type ManageSearchData struct{}
+
+var _ pcap.INewSource = ManageSearchData{}
+var _ pcap.IClear = ManageSearchData{}
+
+// Make sure that existing stream widgets are discarded if the user loads a new pcap.
+func (t ManageSearchData) OnNewSource(c pcap.HandlerCode, app gowid.IApp) {
+	clearSearchData(app)
+}
+
+func (t ManageSearchData) OnClear(c pcap.HandlerCode, app gowid.IApp) {
+	clearSearchData(app)
+}
+
+//======================================================================
+
 type checkGlobalJumpAfterPsml struct {
 	Jump termshark.GlobalJumpPos
 }
@@ -347,7 +367,7 @@ func (s SetStructWidgets) OnError(code pcap.HandlerCode, app gowid.IApp, err err
 	// Hack to avoid picking up errors at other parts of the load
 	// cycle. There should be specific handlers for specific errors.
 	if s.Ld.PdmlLoader.IsLoading() {
-		if !termshark.ConfBool("main.suppress-tshark-errors", false) {
+		if !profiles.ConfBool("main.suppress-tshark-errors", true) {
 			app.Run(gowid.RunFunction(func(app gowid.IApp) {
 				OpenLongError(fmt.Sprintf("%v", err), app)
 			}))
