@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -8,7 +8,6 @@ package ui
 import (
 	"fmt"
 	"os"
-	"strings"
 	"sync"
 	"time"
 
@@ -19,12 +18,13 @@ import (
 	"github.com/gcla/gowid/widgets/null"
 	"github.com/gcla/gowid/widgets/table"
 	"github.com/gcla/termshark/v2"
-	"github.com/gcla/termshark/v2/pcap"
-	"github.com/gcla/termshark/v2/pdmltree"
-	"github.com/gcla/termshark/v2/streams"
+	"github.com/gcla/termshark/v2/configs/profiles"
+	"github.com/gcla/termshark/v2/pkg/pcap"
+	"github.com/gcla/termshark/v2/pkg/pdmltree"
+	"github.com/gcla/termshark/v2/pkg/streams"
 	"github.com/gcla/termshark/v2/widgets/appkeys"
 	"github.com/gcla/termshark/v2/widgets/streamwidget"
-	"github.com/gdamore/tcell"
+	"github.com/gdamore/tcell/v2"
 	lru "github.com/hashicorp/golang-lru"
 	log "github.com/sirupsen/logrus"
 )
@@ -339,27 +339,22 @@ func (t *streamParseHandler) OnError(code pcap.HandlerCode, app gowid.IApp, err
 	if !Running {
 		fmt.Fprintf(os.Stderr, "%v\n", err)
 		RequestQuit()
-	} else if !termshark.ConfBool("main.suppress-tshark-errors", false) {
+	} else if !profiles.ConfBool("main.suppress-tshark-errors", true) {
 		var errstr string
 		if kverr, ok := err.(gowid.KeyValueError); ok {
-			errstr = fmt.Sprintf("%v\n\n", kverr.Cause())
-			kvs := make([]string, 0, len(kverr.KeyVals))
-			for k, v := range kverr.KeyVals {
-				kvs = append(kvs, fmt.Sprintf("%v: %v", k, v))
-			}
-			errstr = errstr + strings.Join(kvs, "\n")
+			errstr = termshark.KeyValueErrorString(kverr)
 		} else {
 			errstr = fmt.Sprintf("%v", err)
 		}
 
 		app.Run(gowid.RunFunction(func(app gowid.IApp) {
-			OpenError(errstr, app)
+			OpenLongError(errstr, app)
 		}))
 	}
 }
 
 func initStreamWidgetCache() {
-	widgetCacheSize := termshark.ConfInt("main.stream-cache-size", 100)
+	widgetCacheSize := profiles.ConfInt("main.stream-cache-size", 100)
 
 	var err error
 	streamWidgets, err = lru.New(widgetCacheSize)
@@ -441,7 +436,7 @@ func makeStreamWidget(previousFilter string, filter string, cap string, proto st
 			MenuOpener: &multiMenu1Opener,
 			DefaultDisplay: func() streamwidget.DisplayFormat {
 				view := streamwidget.Hex
-				choice := termshark.ConfString("main.stream-view", "hex")
+				choice := profiles.ConfString("main.stream-view", "hex")
 				switch choice {
 				case "raw":
 					view = streamwidget.Raw
