@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -10,7 +10,7 @@ import (
 	"testing"
 
 	"github.com/blang/semver"
-	"github.com/gcla/termshark/v2/format"
+	"github.com/gcla/termshark/v2/pkg/format"
 	"github.com/stretchr/testify/assert"
 )
 
@@ -164,6 +164,16 @@ func TestIPComp1(t *testing.T) {
 	assert.False(t, ip.Less("2001:db8::68", "192.168.0.253"))
 }
 
+func TestMACComp1(t *testing.T) {
+	var mac MACCompare
+	assert.True(t, mac.Less("x", "y"))
+	assert.True(t, mac.Less("11:22:33:44:55:66", "y"))
+	assert.True(t, mac.Less("xx:22:33:44:55:66", "y"))
+	assert.False(t, mac.Less("xx:22:33:44:55:66", "11:22:33:44:55:66"))
+	assert.True(t, mac.Less("11:22:33:44:55:66", "11:22:33:44:55:67"))
+	assert.False(t, mac.Less("11:22:33:44:55:66", "11:22:33:44:54:66"))
+}
+
 func TestFolders(t *testing.T) {
 	tmp := os.Getenv("TMPDIR")
 	os.Setenv("TMPDIR", "/foo")
