@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -16,7 +16,7 @@ import (
 	"github.com/gcla/gowid/widgets/list"
 	"github.com/gcla/gowid/widgets/tree"
 	"github.com/gcla/termshark/v2"
-	"github.com/gcla/termshark/v2/pdmltree"
+	"github.com/gcla/termshark/v2/pkg/pdmltree"
 )
 
 //======================================================================
