@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -24,7 +24,7 @@ import (
 	"github.com/gcla/gowid/widgets/styled"
 	"github.com/gcla/gowid/widgets/text"
 	"github.com/gcla/termshark/v2/widgets/keepselected"
-	"github.com/gdamore/tcell"
+	"github.com/gdamore/tcell/v2"
 )
 
 //======================================================================
@@ -457,7 +457,11 @@ func (w *Widget) updateCompletions(app gowid.IApp) {
 func Open(w *Widget, container gowid.ISettableComposite, width gowid.IWidgetDimension, app gowid.IApp) {
 	w.ov = overlay.New(w, container.SubWidget(),
 		gowid.VAlignBottom{}, gowid.RenderWithUnits{U: 3}, // Intended to mean use as much vertical space as you need
-		gowid.HAlignLeft{Margin: 5, MarginRight: 5}, width)
+		gowid.HAlignLeft{Margin: 5, MarginRight: 5}, width,
+		overlay.Options{
+			IgnoreLowerStyle: true,
+		},
+	)
 
 	if _, ok := width.(gowid.IRenderFixed); ok {
 		w.SetContentWidth(gowid.RenderFixed{}, app) // fixed or weight:1, ratio:0.5
