@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source code is governed by the MIT license
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source code is governed by the MIT license
 // that can be found in the LICENSE file.
 
 package number
@@ -8,7 +8,7 @@ import (
 
 	"github.com/gcla/gowid"
 	"github.com/gcla/gowid/gwtest"
-	"github.com/gdamore/tcell"
+	"github.com/gdamore/tcell/v2"
 	log "github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/assert"
 )
