@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -36,8 +36,9 @@ import (
 	"github.com/gcla/gowid/widgets/table"
 	"github.com/gcla/gowid/widgets/text"
 	"github.com/gcla/termshark/v2"
-	"github.com/gcla/termshark/v2/format"
-	"github.com/gcla/termshark/v2/streams"
+	"github.com/gcla/termshark/v2/configs/profiles"
+	"github.com/gcla/termshark/v2/pkg/format"
+	"github.com/gcla/termshark/v2/pkg/streams"
 	"github.com/gcla/termshark/v2/ui/menuutil"
 	"github.com/gcla/termshark/v2/ui/tableutil"
 	"github.com/gcla/termshark/v2/widgets"
@@ -49,7 +50,7 @@ import (
 	"github.com/gcla/termshark/v2/widgets/scrollabletable"
 	"github.com/gcla/termshark/v2/widgets/trackfocus"
 	"github.com/gcla/termshark/v2/widgets/withscrollbar"
-	"github.com/gdamore/tcell"
+	"github.com/gdamore/tcell/v2"
 )
 
 //======================================================================
@@ -416,7 +417,7 @@ func (w *Widget) construct() {
 			for i := 0; i < len(w.tblWidgets); i++ {
 				w.updateChunkModel(i, w.displayAs, app)
 			}
-			termshark.SetConf("main.stream-view", "hex")
+			profiles.SetConf("main.stream-view", "hex")
 		}
 	}})
 	rb2.OnClick(gowid.WidgetCallback{"cb", func(app gowid.IApp, w2 gowid.IWidget) {
@@ -425,7 +426,7 @@ func (w *Widget) construct() {
 			for i := 0; i < len(w.tblWidgets); i++ {
 				w.updateChunkModel(i, w.displayAs, app)
 			}
-			termshark.SetConf("main.stream-view", "ascii")
+			profiles.SetConf("main.stream-view", "ascii")
 		}
 	}})
 	rb3.OnClick(gowid.WidgetCallback{"cb", func(app gowid.IApp, w2 gowid.IWidget) {
@@ -434,7 +435,7 @@ func (w *Widget) construct() {
 			for i := 0; i < len(w.tblWidgets); i++ {
 				w.updateChunkModel(i, w.displayAs, app)
 			}
-			termshark.SetConf("main.stream-view", "raw")
+			profiles.SetConf("main.stream-view", "raw")
 		}
 	}})
 
@@ -901,7 +902,7 @@ func (w *Widget) AddChunkEntire(ch streams.IChunk, app gowid.IApp) {
 		w.updateChunkModel(i, w.displayAs, app)
 	}
 
-	w.doMenuUpdate = true
+	w.updateConvMenuWidget(app)
 
 	w.data.currentChunk++
 }
